// Here goes prototypes declaration and documentation

#include <windows.h>
#ifndef __PCI1032__
	#include "define.h" /* this include file stores typedef that can be easily used */
#endif
#ifdef __cplusplus
	extern "C" {
#endif

/**
 * \defgroup General xPCI-1032 general functions
 * @{
 */

/**  Enables to get the number of xPCI-1032 boards present in the system. \n
 * The function fill the value pointed by pb_NbrOfBoards with the number of boards present.
 *                                                                            
 * @param[out] pb_NbrOfBoards Value that will be filled with the number of boards     
 * @retval	0 : No error
 * @retval -1 : Error while getting the number of boards
 */
_INT_ i_PCI1032_GetNumberOfBoards (PBYTE	pb_NbrOfBoards );


/** Get device information of the board. For the 32 bits filter board this function must be called just after the Open board function in order to set filter property.
 *                                                                   
 * @param[in] b_BoardIndex : Index of the board
 * @param[in] dw_IdentifierStringSize : size of the identifier parameter 
 * @param[out] pc_Identifier: Identifier of the board
 * @param[out] pdw_UINumber : This number is typically a user-perceived slot number, such as a number printed next to the slot on the board, or some other number that makes locating the physical device easier for the user. If the device is on a bus that has no UI number convention, or if the bus driver for the device cannot determine the UI number, this value is 0xFFFFFFFF.
 * @param[out] pdw_DeviceNumber : Device number.
 * @param[out] pdw_BusNumber : Bus number.
 * @param[out] pdw_BaseAddress0 : Base address 0 of the board
 * @param[out] pdw_BaseAddress1 : Base address 1 of the board
 * @param[out] pb_Interrupt : Interrupt line of the board
 * @retval	 0 : No error
 * @retval	-1 : Unable to open the board via index
 * @retval	-2 : Error when calling driver's IOCTL
 */
_INT_ i_PCI1032_GetBoardInformation(BYTE		b_BoardIndex,
									DWORD		dw_IdentifierStringSize,
									PCHAR		pc_Identifier,
									PDWORD		pdw_UINumber,
									PDWORD		pdw_DeviceNumber,
									PDWORD		pdw_BusNumber,
									PDWORD		pdw_BaseAddress0,
									PDWORD		pdw_BaseAddress1,
									PBYTE		pb_Interrupt );


/** Open the board with the index : b_BoardIndex. A handle is returned to the \n    
 * user which allows to use the driver functions. \n 
 *                                                                                  
 * @param[in] b_BoardIndex : Index of the board to open                             
 * @param[out] ph_DeviceHandle : Handle of the selected xPCI-1032 board for using the functions
 * @retval	 0 : No error
 * @retval	-1 : Unable to get the number of available boards
 * @retval	-2 : Wrong device index parameter. The parameter is too high
 * @retval	-3 : Error while getting the device's class
 * @retval	-4 : Error while getting informations about the device
 * @retval	-5 : Memory allocation error
 * @retval	-6 : Get device interface detail error
 * @retval	-7 : Error while creating communication handle
 * @retval	-8 : Error in the initialization callback function
 */
_INT_ i_PCI1032_OpenBoardViaIndex (BYTE	b_BoardIndex,
								   PHANDLE	ph_DeviceHandle );  


/** Open the board with the indentifier : pc_Identifier. A handle is returned to the \n    
 * user which allows to use the following functions. \n                                                         
 *                                                                                  
 * @param[in] pc_Identifier : Identifier of the board to open
 * @param[out] ph_DeviceHandle : Handle of the board xPCI-1032 for using the functions
 * @retval	0 : No Error
 * @retval	-1 : Unable to get the number of available boards
 * @retval	-2 : Unable to get the board identifier
 * @retval	-3 : Unable to open the board
 * @retval	-4 : Identifier not found
 */
_INT_ i_PCI1032_OpenBoardViaIdentifier (PCHAR	pc_Identifier,
										PHANDLE	ph_DeviceHandle );


/** Release the handle of the board. Block the access to the board.
 *                                                                   
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @retval	0 : No Error
 * @retval	-1 : The device handle parameter is wrong
 */
_INT_ i_PCI1032_CloseBoard (HANDLE	h_DeviceHandle );


/** Return the version of the DLL.
 *                                                                   
 * @param[out] pdw_Version  : Version. sample : 0x01010910 -> version 1.01 september 2010       
 * @retval	0 : No Error
 */
_INT_ i_PCI1032_GetDLLVersion (PDWORD	pdw_Version);

/*Function's Name   : i_PCI1032_GetDeviceDriverVersion*/
/** Return the version of the device driver.
*                                                                   
* @param[in] h_DeviceHandle  : Handle of the board                
* @param[out] pdw_Version  : Version. sample : 0x01011211 -> version 1.01 - 12/2011
* @retval	0 : No Error
* @retval	-1 : Error while calling IOCTL function
*/
_INT_ i_PCI1032_GetDeviceDriverVersion(HANDLE h_DeviceHandle, PDWORD pdw_Version);


/**
 * @}
 * \defgroup Interrupt xPCI-1032 interrupt management functions
 * @{
 */

/** Install an interrupt callback function.
 * This function will be called each time the board generates an interrupt. \n
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @param[in] v_FunctionName    : Address of the callback function
 *		\n	\b h_DeviceHandle	: Handle of the board
 *		\n	\b b_FIFOOverflow	: If b_FIFOOverflow != 0, there are too many untreated interrupts in the kernel FIFO, and any new information is lost
 *		\n	\b w_InterruptMask	: Source of the interrupt. It is in fact a mask of channels (on 16 bits since 16 channels 0->15 are interruptible).
 *		\n	\b w_InputValues	: Current status of the digital inputs 0->15
 *
 *
 * @retval	0 : No Error
 * @retval	-1 : The device handle parameter is wrong
 * @retval	-2 : An interrupt routine was still installed. Remove it with i_PCI1032_ResetBoardIntRoutine and retry
 * @retval	-3 : Error while creating kill event
 * @retval	-4 : Error while creating interrupt event
 * @retval	-5 : Error while creating interrupt thread
 * @retval	-6 : Error while changing the priority class of the thread
 * @retval	-7 : Error while changing the priority of the thread
 * @retval	-8 : Error while calling the IOCTL command
 * @retval	-9 : Error in the interrupt callback function
 */
_INT_ i_PCI1032_SetBoardIntRoutine (HANDLE	h_DeviceHandle,
									void	(*v_FunctionName)(	HANDLE	h_DeviceHandle,
																BYTE	b_FIFOOverflow,
																WORD	w_InterruptMask,
																WORD	w_InputValues )
);


/** Install an interrupt callback function.
 * This function will be called each time the board generates an interrupt. \n
 *
 * @param[in] h_DeviceHandle    : Handle of the board
 * @param[in] v_FunctionName    : Address of the callback function
 *		\n	\b h_DeviceHandle	: Handle of the board
 *		\n	\b b_FIFOOverflow	: If b_FIFOOverflow != 0, there are too many untreated interrupts in the kernel FIFO, and any new information is lost
 *		\n	\b dw_InterruptMask	: Source of the interrupt. It is in fact a mask of channels (depends on board type).
 *		\n	\b dw_InputValues	: Current status of the digital inputs 0->31
 *
 *
 * @retval	0 : No Error
 * @retval	-1 : The device handle parameter is wrong
 * @retval	-2 : An interrupt routine was still installed. Remove it with i_PCI1032_ResetBoardIntRoutine and retry
 * @retval	-3 : Error while creating kill event
 * @retval	-4 : Error while creating interrupt event
 * @retval	-5 : Error while creating interrupt thread
 * @retval	-6 : Error while changing the priority class of the thread
 * @retval	-7 : Error while changing the priority of the thread
 * @retval	-8 : Error while calling the IOCTL command
 * @retval	-9 : Error in the interrupt callback function
 * @retval	-10 : This function is not available for this board
 */
_INT_ i_PCI1032_SetBoardIntRoutineEx (HANDLE	h_DeviceHandle,
									void	(*v_FunctionName)(	HANDLE	h_DeviceHandle,
																BYTE	b_FIFOOverflow,
																DWORD	dw_InterruptMask,
																DWORD	dw_InputValues)
);

/** Deinstall the interrupt callback function.
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @retval	0 : No Error
 * @retval	-1 : The device handle parameter is wrong
 * @retval	-2 : No interrupt routine installed
 * @retval	-3 : Error in the reset interrupt callback function
 */
_INT_ i_PCI1032_ResetBoardIntRoutine (HANDLE	h_DeviceHandle );


/** 
 * @}
 * \defgroup DigInput xPCI-1032 digital inputs functions
 * @{
 */

/** Read 1 digital inputs of the board xPCI-1032
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Channel The index of the channel to read (0 to 31)
 * @param[out] pb_Value Status of the digital input
 *
 * @retval	0 : No Error
 * @retval	-1 : The channel index is wrong
 * @retval	-2 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI1032_Read1DigitalInput (HANDLE h_DeviceHandle,
								   BYTE b_Channel,
								   PBYTE pb_Value );


/** Read 8 digital inputs of the board xPCI-1032
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Port Port to read ( \b 0: channels 0-7, \b 1: channels 8-15, \b 2: channels 16-23, \b 3: channels 24-31)
 * @param[out] pb_Value Status of the digital inputs
 *
 * @retval	0 : No Error
 * @retval	-1 : The port index is wrong
 * @retval	-2 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI1032_Read8DigitalInputs (HANDLE	h_DeviceHandle,
									BYTE	b_Port,
									PBYTE	pb_Value );


/** Read 16 digital inputs of the board xPCI-1032
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Port Port to read ( \b 0: channels 0-15, \b 1: channels 15-31)
 * @param[out] pw_Value Status of the digital inputs
 *
 * @retval	0 : No Error
 * @retval	-1 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI1032_Read16DigitalInputs (HANDLE	h_DeviceHandle,
									 BYTE b_Port,
									 PWORD	pw_Value );

/** Read 32 digital inputs of the board xPCI-1032
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pdw_Value Status of the digital inputs
 *
 * @retval	0 : No Error
 * @retval	-1 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI1032_Read32DigitalInputs (HANDLE	h_DeviceHandle,
									 PDWORD	pdw_Value );
/**
 * @}
 * \defgroup Event xPCI-1032 event logic functions
 * @{
 */

/** Permits to initialize the event logic of the board xPCI-1032 \n
 * The event logic enables you to receive interrupt when a certain state is reached by the board. \n
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_EventMode Event logic mode. Values are 0 for OR or 1 for AND.
 * @param[in] pc_EventMask An 16-characters string, to set the mask of the event. \n
 *	Each character gives the state of one input channels and can have one of the following values :
 *	\li "d" : The input is not used for event
 *	\li "h" : The input reacts on high state
 *	\li "l" : The input reacts on low state
 *	\li "b" : The input reacts on both states
 *	\n
 *	From the left to the right, the first digit of the character string is input 15 and the last digit is input 0. \n
 *	@retval  0 : No error
 *	@retval -1 : wrong device handle
 *	@retval -2 : Invalid parameter b_EventMode. Must be 0: OR or 1: AND
 *	@retval -3 : Parameter pc_EventMask is null
 *	@retval -4 : The size of the string pointer by pc_EventMask is not 16
 *	@retval -5 : One of the character of the mask is not valid
 *	@retval -6 : No interrupt routine is installed
 *	@retval -7 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI1032_InitEventLogic (HANDLE	h_DeviceHandle,
								BYTE	b_EventMode,
								PCHAR	pc_EventMask );


/** Starts the event logic of the board xPCI-1032 \n
 * You  must call the function i_PCI1032_InitEventLogic before calling this function
 *
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @retval	0 : No Error
 * @retval	-1 : wrong device handle
 * @retval	-2 : The interrupt routine is not installed
 * @retval	-3 : Error while calling IOCTL function (wrong handle ?)
*/
_INT_ i_PCI1032_StartEventLogic (HANDLE h_DeviceHandle);

	
/** Stops the event logic of the board xPCI-1032
 *
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @retval	0 : No Error
 * @retval	-1 : wrong device handle
 * @retval	-2 : Error while calling IOCTL function (wrong handle ?)
*/
_INT_ i_PCI1032_StopEventLogic (HANDLE h_DeviceHandle);

/**
 * @}
 * \defgroup Filter xPCI-1032 Filter functions
 * @{
 */

/** Init the digital input module filter of the board xPCI-1032.
 * This functionality is not available for all boards.
 * If this function returns -3, it means that the board is not compatible
 *
 *@param[in] h_DeviceHandle Handle of the board
 *@param[in] dw_DelayValue  Filter Delay value, between 0 -> 630 ms with 10ms step
 *
 *@retval 0 : No Error
 *@retval -1 : Wrong device handle
 *@retval -2 : Error while calling IOCTL function (wrong handle ?)
 *@retval -3 : Functionality not available
 *@retval -4 : Delay value error -> must be multiple of 10 and between 0 -> 630
 */
_INT_ i_PCI1032_InitDigitalInputModuleFilter (HANDLE h_DeviceHandle,
											  DWORD  dw_DelayValue);

/** Enable or disable the digital input module filter of the board xPCI-1032
 * This functionality is not available for all boards.
 * If this function returns -3, it means that the board is not compatible.
 *
 *@param[in] h_DeviceHandle Handle of the board
 *@param[in] b_FilterFlag   Status of the digital input module filter
 *							\li 0 Filter is disabled
 *							\li 1 Filter is enabled
 *
 *@retval 0 : No Error
 *@retval -1 : Wrong device handle
 *@retval -2 : Error while calling IOCTL function (wrong handle ?)
 *@retval -3 : Functionality not available
 *@retval -4 : FilterFlag parameter is wrong, must be 0 or 1
 */
_INT_ i_PCI1032_EnableDisableDigitalInputModuleFilter(HANDLE h_DeviceHandle,
													  BYTE   b_FilterFlag);

/**
 * @}
 */
#ifdef __cplusplus
	}
#endif


