using System;
using System.Collections.Generic;
using System.Text;

namespace PCI1032_NET
{
    public class PCI1032
    {
        /// <summary>
        /// Store the count
        /// </summary>
        private static int boardCount;

        /// <summary>
        /// Store the index of the board
        /// </summary>
        private int index;

        /// <summary>
        /// Store an handle to the board. Will be used for interrupt routine for example
        /// </summary>
        private IntPtr handle;

        /// <summary>
        /// Handler to the interrupt function of the board
        /// The function pointed by this delegate will be called each time the board interrupts
        /// </summary>
        private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

        /// <summary>
        /// An object that will contains the event logic event args
        /// Created only once, because the new command takes a lot of resources
        /// </summary>
        private EventLogicEventArgs eventLogicEventArgs;

        /// <summary>
        /// Delegate to provide to listen to the event logic changes
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void EventLogicEventHandler(object source, EventLogicEventArgs args);

        /// <summary>
        /// The event logic event
        /// </summary>
        public event EventLogicEventHandler EventLogicEventReceived;

        /// <summary>
        /// Static constructor to instanciate the class and initialize the variables
        /// </summary>
        static PCI1032()
        {
            PCI1032.boardCount = 0;

            // Get the number of boards
            if ((PCI1032.boardCount = GetBoardNumber()) == -1)
            {
                throw new Exception("PCI1032-GetBoardNumber error:" + PCI1032.boardCount.ToString());
            }
        }

        /// <summary>
        /// Constructor of the board
        /// Cannot be called; initialize the internal variables
        /// </summary>
        private PCI1032()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args
            this.eventLogicEventArgs = new EventLogicEventArgs();

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);
        }

        /// <summary>
        /// Destructor of the board
        /// Free all the ressources
        /// </summary>
        ~PCI1032()
        {
            DllImport.i_PCI1032_CloseBoard(handle);
            this.index = -1;
            this.handle = IntPtr.Zero;
        }

        /// <summary>
        /// Enables to get an instance of a PCI1032 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCI1032 on success</returns>
        public static PCI1032 OpenBoard(int index)
        {
            PCI1032 boardInstance = null;
            IntPtr handle = IntPtr.Zero;

            // Check that the index is in the range
            if (index < 0 || index > PCI1032.boardCount)
            {
                Console.WriteLine("index not valid");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCI1032_OpenBoardViaIndex((byte)index, ref handle) != 0)
            {
                Console.WriteLine("i_PCI1032_OpenBoardViaIndex failed");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCI1032();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Enables to get the number of PCI1032 boards plugged in the computer
        /// </summary>
        /// <returns>The number of board</returns>
        public static int GetBoardNumber()
        {
            int ret = 0;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCI1032_GetNumberOfBoards(ref nbBoard);

            // Error case
            if (ret != 0) 
            {
                return -1; 
            }

             // Success
            else 
            { 
                return (int)nbBoard; 
            }
        }

        #region "Interrupt"

        /// <summary>
        /// Interrupt function. Called each time the board interrupts
        /// </summary>
        /// <param name="handle">handle to the board</param>
        /// <param name="fifoOverflow"></param>
        /// <param name="mask"></param>
        /// <param name="values"></param>
        private void InterruptFunction(IntPtr handle, byte fifoOverflow, ushort mask, ushort values)
        {
            // The only interrupt is the event interrupt
            this.eventLogicEventArgs.FifoOverflow = fifoOverflow;
            this.eventLogicEventArgs.Mask = mask;
            this.eventLogicEventArgs.Values = values;

            // Send the event
            this.OnEventLogicEventReceived(this.eventLogicEventArgs);
        }

        /// <summary>
        /// Enable to use the interrupt of the board
        /// To receive an interrupt, you have to listen to the event of the object
        /// </summary>
        /// <returns></returns>
        public int SetBoardIntRoutine()
        {
            return DllImport.i_PCI1032_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        /// <returns></returns>
        public int ResetBoardIntRoutine()
        {
            return DllImport.i_PCI1032_ResetBoardIntRoutine(this.handle);
        }

        /// <summary>
        /// Enables to signal to all the listeners that an event on the event logic occurs
        /// </summary>
        /// <param name="args"></param>
        protected virtual void OnEventLogicEventReceived(EventLogicEventArgs args)
        {
            if (this.EventLogicEventReceived != null)
            {
                this.EventLogicEventReceived(this, args);
            }
        }

        #endregion

        #region "Digital inputs "

        /// <summary>
        /// Enables to read one digital input
        /// </summary>
        /// <param name="channel">Channel to read (0-31)</param>
        /// <param name="inputValue">A byte that will be filled with the current status of the input (0,1)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read1DigitalInput(byte channel, ref byte inputValue)
        {
            // Call the function
            return DllImport.i_PCI1032_Read1DigitalInput(this.handle, channel, ref inputValue);
        }

        /// <summary>
        /// Enables to read 8 digital inputs (per port)
        /// </summary>
        /// <param name="port">The port you want to read. 0 is for the input 0 to 7, 1 for the inputs 8 to 15, 2 for the inputs 16 to 23 and 3 for the inputs 24 to 31 (0-3)</param>
        /// <param name="inputValues">A byte that will be filled with the current status of the port</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read8DigitalInputs(byte port, ref byte inputValues)
        {
            // Call the function
            return DllImport.i_PCI1032_Read8DigitalInputs(this.handle, port, ref inputValues);
        }

        /// <summary>
        /// Enables to read 16 digital inputs of the board
        /// </summary>
        /// <param name="port">he port you want to read. 0 is for the input 0 to 15, and 1 for the inputs 16 to 31 (0,1)</param>
        /// <param name="value">A ushort that will be filled with the current status of the inputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read16DigitalInputs(byte port, ref ushort inputValues)
        {
            // Call the function
            return DllImport.i_PCI1032_Read16DigitalInputs(this.handle, port, ref inputValues);
        }

        /// <summary>
        /// Read all the digital inputs of the board
        /// </summary>
        /// <param name="inputValues">A uint that will be filled with the status of the inputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read32DigitalInputs(ref uint inputValues)
        {
            // Call the function
            return DllImport.i_PCI1032_Read32DigitalInputs(this.handle, ref inputValues);
        }

        #endregion

        #region "Event logic"

        /// <summary>
        /// Initialize the event logic
        /// See doxygen documentation
        /// </summary>
        /// <param name="eventLogic"></param>
        /// <param name="eventMask"></param>
        /// <returns></returns>
        public int InitEventLogic(byte eventMode, string eventMask)
        {
            // Call the function
            return DllImport.i_PCI1032_InitEventLogic(this.handle, eventMode, eventMask);
        }

        /// <summary>
        /// Enables to start the event logic
        /// </summary>
        /// <returns></returns>
        public int StartEventLogic()
        {
            // Call the function
            return DllImport.i_PCI1032_StartEventLogic(this.handle);
        }

        /// <summary>
        /// Enables to stop the event logic
        /// </summary>
        /// <returns></returns>
        public int StopEventLogic()
        {
            // Call the function
            return DllImport.i_PCI1032_StopEventLogic(this.handle);
        }

        #endregion
    }
}
